@@ -38,12 +38,14 @@
 
 #include <functional>
 #include <unistd.h>
+#include <cstdio>
 #include <rak/file_stat.h>
 #include <rak/error_number.h>
 #include <rak/path.h>
 #include <rak/socket_address.h>
 #include <rak/string_manip.h>
 #include <torrent/rate.h>
+#include <torrent/throttle.h>
 #include <torrent/tracker.h>
 #include <torrent/connection_manager.h>
 #include <torrent/data/file.h>
@@ -252,6 +254,49 @@ retrieve_d_local_id_html(core::Download* download) {
   return torrent::Object(rak::copy_escape_html(hashString->begin(), hashString->end()));
 }
 
+torrent::Object
+apply_d_custom(core::Download* download, const torrent::Object& rawArgs) {
+  torrent::Object::list_const_iterator itr = rawArgs.as_list().begin();
+  if (itr == rawArgs.as_list().end())
+    throw torrent::bencode_error("Missing key argument.");
+
+  const std::string& key = itr->as_string();
+  if (++itr == rawArgs.as_list().end())
+    throw torrent::bencode_error("Missing value argument.");
+
+  download->bencode()->get_key("rtorrent").
+                       insert_preserve_copy("custom", torrent::Object::create_map()).first->second.
+                       insert_key(key, itr->as_string());
+  return torrent::Object();
+}
+
+torrent::Object
+retrieve_d_custom(core::Download* download, const std::string& key) {
+  try {
+    return download->bencode()->get_key("rtorrent").get_key("custom").get_key_string(key);
+
+  } catch (torrent::bencode_error& e) {
+    return std::string();
+  }
+}
+
+torrent::Object
+retrieve_d_custom_throw(core::Download* download, const std::string& key) {
+  try {
+    return download->bencode()->get_key("rtorrent").get_key("custom").get_key_string(key);
+
+  } catch (torrent::bencode_error& e) {
+    throw torrent::input_error("No such custom value.");
+  }
+}
+
+torrent::Object
+retrieve_d_bitfield(core::Download* download) {
+  const torrent::Bitfield* bitField = download->download()->file_list()->bitfield();
+
+  return torrent::Object(rak::transform_hex(bitField->begin(), bitField->end()));
+}
+
 // Just a helper function atm.
 torrent::Object
 cmd_d_initialize_logs(core::Download* download) {
@@ -515,6 +560,7 @@ initialize_command_download() {
   ADD_CD_VOID("hash",          &retrieve_d_hash);
   ADD_CD_VOID("local_id",      &retrieve_d_local_id);
   ADD_CD_VOID("local_id_html", &retrieve_d_local_id_html);
+  ADD_CD_VOID("bitfield",      &retrieve_d_bitfield);
   ADD_CD_VOID("base_path",     &retrieve_d_base_path);
   ADD_CD_VOID("base_filename", &retrieve_d_base_filename);
   ADD_CD_STRING_UNI("name",    rak::on(std::mem_fun(&core::Download::download), std::mem_fun(&torrent::Download::name)));
@@ -560,6 +606,10 @@ initialize_command_download() {
   ADD_CD_VARIABLE_STRING_PUBLIC("custom4", "rtorrent", "custom4");
   ADD_CD_VARIABLE_STRING_PUBLIC("custom5", "rtorrent", "custom5");
 
+  ADD_CD_SLOT_PUBLIC("d.set_custom",       call_list,   rak::ptr_fn(&apply_d_custom), "i:", "");
+  ADD_CD_SLOT_PUBLIC("d.get_custom",       call_string, rpc::object_string_fn<core::Download*>(std::ptr_fun(&retrieve_d_custom)), "s:s", "");
+  ADD_CD_SLOT_PUBLIC("d.get_custom_throw", call_string, rpc::object_string_fn<core::Download*>(std::ptr_fun(&retrieve_d_custom_throw)), "s:s", "");
+
   // 0 - stopped
   // 1 - started
   ADD_CD_VARIABLE_VALUE("state", "rtorrent", "state");
@@ -627,6 +677,9 @@ initialize_command_download() {
   ADD_CD_VALUE_MEM_UNI("skip_rate",    &torrent::Download::mutable_skip_rate, &torrent::Rate::rate);
   ADD_CD_VALUE_MEM_UNI("skip_total",   &torrent::Download::mutable_skip_rate, &torrent::Rate::total);
 
+  ADD_CD_STRING("set_throttle_name",      std::mem_fun(&core::Download::set_throttle_name));
+  ADD_CD_SLOT_PUBLIC("d.get_throttle_name", call_unknown, rpc::get_variable_d_fn("rtorrent", "throttle_name"), "i:", "");
+
   ADD_CD_VALUE_UNI("creation_date",       rak::on(std::mem_fun(&core::Download::download), std::mem_fun(&torrent::Download::creation_date)));
   ADD_CD_VALUE_UNI("bytes_done",          rak::on(std::mem_fun(&core::Download::download), std::mem_fun(&torrent::Download::bytes_done)));
   ADD_CD_VALUE_UNI("ratio",               std::ptr_fun(&retrieve_d_ratio));
