@@ -50,7 +50,6 @@ namespace ui {
 namespace core {
   class Manager;
   class ViewManager;
-  class Scheduler;
   class DhtManager;
 }
 
@@ -90,7 +89,6 @@ public:
 
   core::Manager*      core()                        { return m_core; }
   core::ViewManager*  view_manager()                { return m_viewManager; }
-  core::Scheduler*    scheduler()                   { return m_scheduler; }
   core::DhtManager*   dht_manager()                 { return m_dhtManager; }
 
   torrent::Poll*      poll();
@@ -120,7 +118,6 @@ private:
 
   core::Manager*      m_core;
   core::ViewManager*  m_viewManager;
-  core::Scheduler*    m_scheduler;
   core::DhtManager*   m_dhtManager;
 
   ui::Root*           m_ui;
